#!/usr/bin/env python3

import os
import sys
import shutil
import subprocess


def fix_binary(path, frameworks_dir):
    print("fixing", path)
    changes = 0
    if not os.path.exists(path):
        raise Exception("could not find " + repr(path))
    args = ["otool", "-L", path]
    p = subprocess.Popen(args, stdout=subprocess.PIPE)
    # noinspection PyUnresolvedReferences
    data = p.stdout.read().decode("UTF-8")
    p.wait()
    for line in data.split("\n"):
        line = line.strip()
        if not line:
            continue
        if line.startswith("/usr/lib") or line.startswith("/System"):
            # old = line.split(" ")[0]
            continue
        if line.startswith("@executable_path"):
            continue
        old = line.split(" ")[0]
        if "Contents" in old:
            continue
        print(old)
        old_dir, name = os.path.split(old)
        new = old.replace(old, "@executable_path/../Frameworks/" + name)
        dst = os.path.join(frameworks_dir, os.path.basename(old))
        if not os.path.exists(dst):
            print("copying", old)
            shutil.copy(old, dst)
            os.chmod(dst, 0o644)
            changes += 1
        if os.path.basename(path) == os.path.basename(old):
            args = ["install_name_tool", "-id", new, path]
        else:
            args = ["install_name_tool", "-change", old, new, path]
        print(args)
        p = subprocess.Popen(args)
        assert p.wait() == 0
    return changes


def fix_iteration(app):
    binaries = []
    mac_os_dir = os.path.join(app, "Contents", "MacOS")
    frameworks_dir = os.path.join(app, "Contents", "Frameworks")
    for name in os.listdir(mac_os_dir):
        binaries.append(os.path.join(mac_os_dir, name))
    for name in os.listdir(frameworks_dir):
        binaries.append(os.path.join(frameworks_dir, name))
    changes = 0
    for binary in binaries:
        changes += fix_binary(binary, frameworks_dir)
    return changes


def main():
    app = sys.argv[1]
    while True:
        changes = fix_iteration(app)
        if changes == 0:
            break


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import os
import sys
import platform
import subprocess

p = subprocess.Popen(["file", "-L", "/bin/sh"], stdout=subprocess.PIPE)
exe_info = p.stdout.read().decode("UTF-8")
if "386" in exe_info:
    arch = "x86"
elif "x86-64" in exe_info:
    arch = "x86-64"
else:
    raise Exception("unrecognized arch " + repr(exe_info))

if os.environ.get("STEAMOS", ""):
    os_name = "steamos"
else:
    os_name = "linux"
    os_name_pretty = "Linux"

version = sys.argv[1]
package_name = "fs-uae_{0}_{1}_{2}".format(version, os_name, arch)
package_name_2 = "FS-UAE_{0}_{1}_{2}".format(version, os_name_pretty, arch)
package_dir = "../{}/FS-UAE/{}/{}".format(os_name, os_name_pretty, arch)


def s(command):
    c = command.format(**globals())
    print(c)
    assert os.system(c) == 0


def wrap(name, target=None, args=None):
    if target is None:
        target = name + ".bin"
        os.rename(os.path.join(package_dir, name),
                  os.path.join(package_dir, target))
    if args is None:
        args = ["$@"]
    path = os.path.join(package_dir, name)
    with open(path, "w") as f:
        f.write("#!/bin/sh\n")
        f.write("MYDIR=$(dirname \"$0\")\n")
        # f.write("export LD_LIBRARY_PATH=\"$MYDIR:$LD_LIBRARY_PATH\"\n")
        command = "\"$MYDIR/{0}\"".format(target)
        for arg in args:
            command += " \"{0}\"".format(arg)
        if os_name == "steamos":
            # if arch == "i686":
            if arch == "x86":
                bin_dir = "bin32"
            # elif arch == "amd64":
            elif arch == "x86-64":
                bin_dir = "bin32"
            else:
                raise Exception("unsupported steamos arch?")
            f.write("if [ -e \"$HOME/.steam/{0}/steam-runtime/"
                    "run.sh\" ]; then\n".format(bin_dir))
            f.write("RUN_SH=\"$HOME/.steam/{0}/steam-runtime/"
                    "run.sh\"\n".format(bin_dir))
            f.write("else\n")
            f.write("RUN_SH=\"/home/steam/.steam/{0}/steam-runtime/"
                    "run.sh\"\n".format(bin_dir))
            f.write("fi\n")
            f.write("exec $RUN_SH {0}\n".format(command))
        else:
            f.write("exec {0}\n".format(command))
    os.chmod(path, 0o755)


s("rm -Rf FS-UAE")
s("mkdir -p {package_dir}")
if os.environ.get("BUILD") == "0":
    pass
else:
    s("cd ../.. && ./configure")
    s("make -C ../..")

s("cp -a ../../fs-uae {package_dir}/fs-uae")
# s("cp -a ../../fs-uae.dat {package_dir}/fs-uae.dat")
# s("PYTHONPATH=../../../fs-uae-launcher:../../../../fs-uae-launcher "
#   "python3 -m fspy.zipfile deterministic {package_dir}/fs-uae.dat")
s("cp -a ../../fs-uae-device-helper {package_dir}/fs-uae-device-helper")

s("mkdir FS-UAE/Data")
s("cp -a ../../fs-uae.dat FS-UAE/Data/fs-uae.dat")
# FIXME: Disabled for now since it references another project
# s("PYTHONPATH=../../../fs-uae-launcher:../../../../fs-uae-launcher "
#   "python3 -m fspy.zipfile deterministic FS-UAE/Data/fs-uae.dat")
# s("mkdir -p {package_dir}/share")
# s("cp -a ../../share/locale {package_dir}/share/locale")
s("cp -a ../../share/locale FS-UAE/Data/Locale")
# s("mkdir -p {package_dir}/share/fs-uae")
# s("touch {package_dir}/share/fs-uae/share-dir")

s("cp -a ../../licenses FS-UAE/Licenses")
s("cp -a ../../README FS-UAE/ReadMe.txt")
s("./standalone-linux.py --strip --rpath='$ORIGIN' {package_dir}")
s("find {package_dir} -name '*.standalone' -delete")
s("echo {version} > FS-UAE/Version.txt")
s("echo {version} > {package_dir}/Version.txt")

if os_name == "steamos":
    wrap("fs-uae")
    wrap("fs-uae-device-helper")

if os.environ.get("PACKAGE", "") != "0":
    # s("cd {package_dir} && tar Jcfv ../../../{package_name}.tar.xz *")
    s("tar Jcfv ../../{package_name}.tar.xz FS-UAE")
    print(package_name)
    s("cp ../../{package_name}.tar.xz ../../{package_name_2}.tar.xz")
    print(package_name_2)
print("OK")

#s("rm -Rf {dbg_package_dir}")
#s("mkdir {dbg_package_dir}")
#s("cp -a ../../fs-uae.dbg {dbg_package_dir}/")
#s("cp -a ../../fs-uae-device-helper.dbg {dbg_package_dir}/")
#s("cd {dbg_package_dir} && tar Jcfv ../../../{dbg_package_dir}.tar.xz *")



#!/usr/bin/env python3

import os
import sys
import time
import subprocess

if os.environ.get("SIGN", "") == "0":
    print("SIGN=0 via environment, skipping signing step")
    sys.exit(0)
if not os.environ.get("APPLE_ID_USER", ""):
    print("APPLE_ID_USER not specified, skipping signing step")
    sys.exit(0)

do_sign = True
tries = 0

while do_sign:
    args = [
        "codesign",
        "-f",
        "--deep",
        "--options=runtime",
        "--entitlements",
        "Entitlements.plist",
        "-s",
        "Developer ID Application",
        sys.argv[1],
    ]
    print(args, "tries", tries)
    p = subprocess.Popen(args)
    if p.wait() == 0:
        break
    else:
        tries += 1
        time.sleep(tries)
        if tries == 20:
            print("Giving up")
            sys.exit(1)
        continue



#!/usr/bin/env python3
import sys
import platform
machine = platform.machine().lower()
if machine in ["x86_64", "amd64", "i386", "i486", "i585", "i686"]:
    if platform.architecture()[0] == "32bit":
        print("x86")
        sys.exit(0)
    if platform.architecture()[0] == "64bit":
        print("x86-64")
        sys.exit(0)
if machine.startswith("power"):
    if platform.architecture()[0] == "32bit":
        print("PPC")
        sys.exit(0)
if machine == "arm64":
    print("ARM64")
    sys.exit(0)
raise Exception(f"Unknown platform for machine {machine}")



#!/usr/bin/env python3

import os
import sys
import subprocess

program_files = os.getenv("ProgramFiles(x86)", "")
if not program_files:
    program_files = os.getenv("PROGRAMFILES")
iscc = os.path.join(program_files, "Inno Setup 5", "iscc.exe")
if not os.path.exists(iscc):
    iscc = os.path.join("C:", "Program Files (x86)", "Inno Setup 5",
                        "iscc.exe")

p = subprocess.Popen([iscc, sys.argv[1]])
sys.exit(p.wait())



#!/usr/bin/env python3

import os
import sys
import subprocess

if os.getenv("MSYSTEM") == "MINGW32":
    mingw_dir = "/mingw32"
elif os.getenv("MSYSTEM") == "MINGW64":
    mingw_dir = "/mingw64"

p = subprocess.Popen([
    "sh", "-c",
    "cp {}/bin/{} {}".format(mingw_dir, sys.argv[1], sys.argv[2])])
sys.exit(p.wait())



#!/usr/bin/env python3

import os
import sys
import time
import xml.etree.ElementTree as ET

if os.environ.get("SIGN", "") == "0":
    print("SIGN=0 via environment, skipping notarization step")
    sys.exit(0)
if os.environ.get("NOTARIZE", "") == "0":
    print("NOTARIZE=0 via environment, skipping notarization step")
    sys.exit(0)
if not os.environ.get("APPLE_ID_USER", ""):
    print("APPLE_ID_USER not specified, skipping notarization step")
    sys.exit(0)

app = sys.argv[1]
app_id = sys.argv[2]

apple_id_user = os.environ.get("APPLE_ID_USER", "")
itc_provider = os.environ.get("ITC_PROVIDER", "")
if os.path.exists(".env"):
    f = open(".env", "r")
    for line in f:
        line = line.strip()
        if not line:
            continue
        key, value = line.split("=")
        os.environ[key] = value
        if key == "APPLE_ID_USER":
            apple_id_user = value
        if key == "ITC_PROVIDER":
            itc_provider = value


def shell(cmd):
    print(cmd)
    return os.popen(cmd).read()


shell("rm -f Notarize.zip")
shell("zip -r Notarize.zip \"{}\"".format(app))
result = shell("xcrun altool --notarize-app -t osx -f Notarize.zip " \
"--primary-bundle-id {} -u {} -p @env:APPLE_ID_PASS " \
"-itc_provider {} --output-format xml".format(
    app_id, apple_id_user, itc_provider))

print(result)
root = ET.fromstring(result)
dict = root.find("dict")
print(dict)
request_uuid = dict.find("dict").find("string").text
print(request_uuid)

while True:
    time.sleep(10.0)
    result = shell("xcrun altool --notarization-info {} " \
    "-u {} -p @env:APPLE_ID_PASS " \
    "-itc_provider {} --output-format xml".format(
        request_uuid, apple_id_user, itc_provider))
    if "<string>success</string>" in result:
        break
    elif "<string>in progress</string>" in result:
        print("in progress...")
        continue
    else:
        print(result)
        raise Exception("...")

print("xcrun stapler staple \"{}\"".format(app))
assert os.system("xcrun stapler staple \"{}\"".format(app)) == 0
shell("rm Notarize.zip")



#!/usr/bin/env python3
import os
import sys
import shutil
import subprocess

strip = False
rpath = False
no_copy = False
# steam_runtime = os.getenv("STEAM_RUNTIME", "")
steam_runtime = os.getenv("STEAMOS", "")


def fix_binary(path):
    changes = 0
    if os.path.exists(path + ".standalone"):
        return changes
    if not os.path.exists(path):
        raise Exception("could not find " + repr(path))

    # find library locations
    args = ["ldd", path]
    p = subprocess.Popen(args, stdout=subprocess.PIPE)
    # noinspection PyUnresolvedReferences
    data = p.stdout.read().decode("UTF-8")
    if p.wait() != 0:
        return 0
    print("fixing", path, "no_copy =", no_copy)
    library_locations = {}
    for line in data.split("\n"):
        line = line.strip()
        if "=>" not in line:
            continue
        library_locations[line.split(" ")[0]] = line.split(" ")[2]

    # find direct dependencies
    args = ["objdump", "-p", path]
    p = subprocess.Popen(args, stdout=subprocess.PIPE)
    # noinspection PyUnresolvedReferences
    data = p.stdout.read().decode("UTF-8")
    if p.wait() != 0:
        return 0
    for line in data.split("\n"):
        line = line.strip()
        # print(line)
        if not line.startswith("NEEDED"):
            continue
        print(line)
        library = line.split(" ")[-1]
        print(library)
        if ignore_library(library):
            continue
        library_source = library_locations[library]
        library_source = os.path.normpath(library_source)
        print(library, library_source)
        # if steam_runtime and library_source.startswith(steam_runtime):
        # if steam_runtime and not library_source.startswith("/usr/local"):
        if steam_runtime and not library_source.startswith("/home"):
            print("skipping steam runtime library")
            continue
        if no_copy:
            print("no_copy is set")
            continue
        dst = os.path.join(os.path.dirname(path), library)
        if not os.path.exists(dst):
            print("copying", library)
            shutil.copy(library_source, dst)
            os.chmod(dst, 0o644)
            changes += 1
    if strip:
        # strip does not work after patchelf has been run
        command = "strip '{}'".format(path)
        print(command)
        os.system(command)
    if rpath:
        command = "patchelf --set-rpath '{}' '{}'".format(rpath, path)
        print(command)
        assert os.system(command) == 0
    # to make sure strip is not run again
    os.system("touch '{}.standalone'".format(path))
    return changes


def ignore_library(name):
    if name.startswith("libgpg-error.so"):
        raise Exception(
            "Bundling libgpg-error (libgcrypt?) breaks Intel GL driver")

    if name.startswith("linux-gate.so"):
        return True
    if name.startswith("linux-vdso.so"):
        return True
    if name.startswith("ld-linux.so.2"):
        return True
    if name.startswith("ld-linux-x86-64.so"):
        return True

    if name.startswith("libc.so"):
        return True
    if name.startswith("libstdc++.so"):
        # Including libstdc++.sp breaks libGL loading with Intel on Ubuntu 16.10
        # libGL error: unable to load driver: i965_dri.so
        return True
    if name.startswith("libgcc_s.so"):
        # Might as well skip this one also, to avoid potential similar problems.
        return True
    if name.startswith("libpthread.so"):
        return True
    if name.startswith("libm.so"):
        return True
    if name.startswith("libdl.so"):
        return True
    if name.startswith("libresolv.so"):
        return True
    if name.startswith("librt.so"):
        return True
    if name.startswith("libutil.so"):
        return True
    # if name.startswith("libpcre.so"):
    #     # Problem with OpenAL on Ubuntu 16.04 if this is included.
    #     return True

    if name.startswith("libGL.so"):
        return True
    if name.startswith("libGLU.so"):
        return True
    if name.startswith("libEGL.so"):
        return True

    if name.startswith("libasound.so"):
        # Alsa library is in LSB, looks like only "old" interfaces are used
        # by SDL2.
        return True

    if name.startswith("libX11.so"):
        return True
    if name.startswith("libXext.so"):
        return True
    if name.startswith("libXcursor.so"):
        return True
    if name.startswith("libXinerama.so"):
        return True
    if name.startswith("libXi.so"):
        return True
    if name.startswith("libXrandr.so"):
        return True
    if name.startswith("libXss.so"):
        return True
    if name.startswith("libXxf86vm.so"):
        return True
    # if name.startswith("libxkbcommon.so"):
    #     return True
    if name.startswith("libxcb.so"):
        return True

    return False


def fix_iteration(app):
    binaries = []
    binaries_dir = app
    for name in os.listdir(binaries_dir):
        binaries.append(os.path.join(binaries_dir, name))
    changes = 0
    for binary in binaries:
        changes += fix_binary(binary)
    return changes


def main():
    global no_copy, strip, rpath
    for arg in list(sys.argv):
        if arg.startswith("--rpath="):
            sys.argv.remove(arg)
            rpath = arg[8:]
        elif arg == "--no-copy":
            sys.argv.remove("--no-copy")
            no_copy = True
        elif arg == "--strip":
            sys.argv.remove("--strip")
            strip = True
    app = sys.argv[1]
    while True:
        changes = fix_iteration(app)
        if changes == 0:
            break
    for name in os.listdir(app):
        if name.endswith(".standalone"):
            os.remove(os.path.join(app, name))


if __name__ == "__main__":
    main()
